@@ -1,4 +1,4 @@
-# Copyright (C) 2018  The Software Heritage developers
+# Copyright (C) 2018-2020  The Software Heritage developers
 # See the AUTHORS file at the top-level directory of this distribution
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
@@ -14,20 +14,27 @@
 
 import collections
 
+from typing import Any, Dict
+
 from swh.model.hashutil import hash_to_bytes
 from swh.model.identifiers import directory_identifier
 
+from swh.storage.interface import StorageInterface
+
 from .dir_iterators import DirectoryIterator, DoubleDirectoryIterator, Remaining
 
+
 # get the hash identifier for an empty directory
 _empty_dir_hash = hash_to_bytes(directory_identifier({"entries": []}))
 
 
-def _get_rev(storage, rev_id):
+def _get_rev(storage: StorageInterface, rev_id: bytes) -> Dict[str, Any]:
     """
     Return revision data from swh storage.
     """
-    return list(storage.revision_get([rev_id]))[0]
+    revision = storage.revision_get([rev_id])[0]
+    assert revision is not None
+    return revision.to_dict()
 
 
 class _RevisionChangesList(object):
@@ -314,7 +321,7 @@ def diff_directories(storage, from_dir, to_dir, track_renaming=False):
     between them.
 
     Args:
-        storage (swh.storage.storage.Storage): instance of a swh
+        storage (swh.storage.interface.StorageInterface): instance of a swh
             storage (either local or remote, for optimal performance
             the use of a local storage is recommended)
         from_dir (bytes): the swh identifier of the directory to compare from
@@ -367,7 +374,7 @@ def diff_revisions(storage, from_rev, to_rev, track_renaming=False):
     i.e. the list of file changes between the two associated directories.
 
     Args:
-        storage (swh.storage.storage.Storage): instance of a swh
+        storage (swh.storage.interface.StorageInterface): instance of a swh
             storage (either local or remote, for optimal performance
             the use of a local storage is recommended)
         from_rev (bytes): the identifier of the revision to compare from
@@ -395,7 +402,7 @@ def diff_revision(storage, revision, track_renaming=False):
     specific revision.
 
     Args:
-        storage (swh.storage.storage.Storage): instance of a swh
+        storage (swh.storage.interface.StorageInterface): instance of a swh
             storage (either local or remote, for optimal performance
             the use of a local storage is recommended)
         revision (bytes): the identifier of the revision from which to
