@@ -8,6 +8,7 @@ import datetime
 from enum import Enum
 from typing import Any, Dict, Iterable, List, Optional, Tuple, TypeVar, Union
 
+from typing_extensions import Protocol, TypedDict, runtime_checkable
 
 from swh.core.api import remote_api_endpoint
 from swh.core.api.classes import PagedResult as CorePagedResult
@@ -22,6 +23,7 @@ from swh.model.model import (
     Release,
     Snapshot,
     SkippedContent,
+    SnapshotBranch,
     MetadataAuthority,
     MetadataAuthorityType,
     MetadataFetcher,
@@ -39,6 +41,19 @@ class ListOrder(Enum):
     DESC = "desc"
 
 
+class PartialBranches(TypedDict):
+    """Type of the dictionary returned by snapshot_get_branches"""
+
+    id: Sha1Git
+    """Identifier of the snapshot"""
+    branches: Dict[bytes, Optional[SnapshotBranch]]
+    """A dict of branches contained in the snapshot
+    whose keys are the branches' names"""
+    next_branch: Optional[bytes]
+    """The name of the first branch not returned or :const:`None` if
+    the snapshot has less than the request number of branches."""
+
+
 TResult = TypeVar("TResult")
 PagedResult = CorePagedResult[TResult, str]
 
@@ -52,7 +67,8 @@ def deprecated(f):
     return f
 
 
-class StorageInterface:
+@runtime_checkable
+class StorageInterface(Protocol):
     @remote_api_endpoint("check_config")
     def check_config(self, *, check_write: bool) -> bool:
         """Check that the storage is configured and ready to go."""
@@ -474,16 +490,14 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("revision")
-    def revision_get(
-        self, revisions: List[Sha1Git]
-    ) -> Iterable[Optional[Dict[str, Any]]]:
+    def revision_get(self, revision_ids: List[Sha1Git]) -> List[Optional[Revision]]:
         """Get revisions from storage
 
         Args:
             revisions: revision ids
 
-        Yields:
-            revisions as dictionaries (or None if the revision doesn't exist)
+        Returns:
+            list of revision object (if the revision exists or None otherwise)
 
         """
         ...
@@ -572,17 +586,15 @@ class StorageInterface:
         ...
 
     @remote_api_endpoint("release")
-    def release_get(
-        self, releases: List[Sha1Git]
-    ) -> Iterable[Optional[Dict[str, Any]]]:
+    def release_get(self, releases: List[Sha1Git]) -> List[Optional[Release]]:
         """Given a list of sha1, return the releases's information
 
         Args:
             releases: list of sha1s
 
-        Yields:
-            dicts with the same keys as those given to `release_add`
-            (or ``None`` if a release does not exist)
+        Returns:
+            List of releases matching the identifiers or None if the release does
+            not exist.
 
         """
         ...
@@ -668,39 +680,10 @@ class StorageInterface:
         """
         ...
 
-    @remote_api_endpoint("snapshot/by_origin_visit")
-    def snapshot_get_by_origin_visit(
-        self, origin: str, visit: int
-    ) -> Optional[Dict[str, Any]]:
-        """Get the content, possibly partial, of a snapshot for the given origin visit
-
-        The branches of the snapshot are iterated in the lexicographical
-        order of their names.
-
-        .. warning:: At most 1000 branches contained in the snapshot will be
-            returned for performance reasons. In order to browse the whole
-            set of branches, the method :meth:`snapshot_get_branches`
-            should be used instead.
-
-        Args:
-            origin: origin identifier (url)
-            visit: the visit identifier
-
-        Returns:
-            dict: None if the snapshot does not exist;
-              a dict with three keys otherwise:
-                * **id**: identifier of the snapshot
-                * **branches**: a dict of branches contained in the snapshot
-                  whose keys are the branches' names.
-                * **next_branch**: the name of the first branch not returned
-                  or :const:`None` if the snapshot has less than 1000
-                  branches.
-
-        """
-        ...
-
     @remote_api_endpoint("snapshot/count_branches")
-    def snapshot_count_branches(self, snapshot_id: Sha1Git) -> Optional[Dict[str, int]]:
+    def snapshot_count_branches(
+        self, snapshot_id: Sha1Git
+    ) -> Optional[Dict[Optional[str], int]]:
         """Count the number of branches in the snapshot with the given id
 
         Args:
@@ -720,7 +703,7 @@ class StorageInterface:
         branches_from: bytes = b"",
         branches_count: int = 1000,
         target_types: Optional[List[str]] = None,
-    ) -> Optional[Dict[str, Any]]:
+    ) -> Optional[PartialBranches]:
         """Get the content, possibly partial, of a snapshot with the given id
 
         The branches of the snapshot are iterated in the lexicographical
@@ -1202,61 +1185,6 @@ class StorageInterface:
         """
         ...
 
-    @deprecated
-    @remote_api_endpoint("algos/diff_directories")
-    def diff_directories(self, from_dir, to_dir, track_renaming=False):
-        """Compute the list of file changes introduced between two arbitrary
-        directories (insertion / deletion / modification / renaming of files).
-
-        Args:
-            from_dir (bytes): identifier of the directory to compare from
-            to_dir (bytes): identifier of the directory to compare to
-            track_renaming (bool): whether or not to track files renaming
-
-        Returns:
-            A list of dict describing the introduced file changes
-            (see :func:`swh.storage.algos.diff.diff_directories`
-            for more details).
-        """
-        ...
-
-    @deprecated
-    @remote_api_endpoint("algos/diff_revisions")
-    def diff_revisions(self, from_rev, to_rev, track_renaming=False):
-        """Compute the list of file changes introduced between two arbitrary
-        revisions (insertion / deletion / modification / renaming of files).
-
-        Args:
-            from_rev (bytes): identifier of the revision to compare from
-            to_rev (bytes): identifier of the revision to compare to
-            track_renaming (bool): whether or not to track files renaming
-
-        Returns:
-            A list of dict describing the introduced file changes
-            (see :func:`swh.storage.algos.diff.diff_directories`
-            for more details).
-        """
-        ...
-
-    @deprecated
-    @remote_api_endpoint("algos/diff_revision")
-    def diff_revision(self, revision, track_renaming=False):
-        """Compute the list of file changes introduced by a specific revision
-        (insertion / deletion / modification / renaming of files) by comparing
-        it against its first parent.
-
-        Args:
-            revision (bytes): identifier of the revision from which to
-                compute the list of files changes
-            track_renaming (bool): whether or not to track files renaming
-
-        Returns:
-            A list of dict describing the introduced file changes
-            (see :func:`swh.storage.algos.diff.diff_directories`
-            for more details).
-        """
-        ...
-
     @remote_api_endpoint("clear/buffer")
     def clear_buffers(self, object_types: Optional[List[str]] = None) -> None:
         """For backend storages (pg, storage, in-memory), this is a noop operation. For proxy
