@@ -3,8 +3,9 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-from typing import Dict, Generator, Iterable
+from typing import Dict, Iterable, Optional
 
+from swh.storage.interface import Sha1
 from swh.model.model import Content, MissingData
 from swh.objstorage import get_objstorage
 from swh.objstorage.exc import ObjNotFoundError
@@ -17,29 +18,31 @@ class ObjStorage:
     the objstorage.
 
     """
+
     def __init__(self, objstorage_config: Dict):
         self.objstorage = get_objstorage(**objstorage_config)
 
     def __getattr__(self, key):
-        if key == 'objstorage':
+        if key == "objstorage":
             raise AttributeError(key)
         return getattr(self.objstorage, key)
 
-    def content_get(self, contents: Iterable[bytes]) -> Generator:
-        """Retrieve content data from the objstorage
+    def content_get(self, obj_id: Sha1) -> Optional[bytes]:
+        """Retrieve data associated to the content from the objstorage
 
         Args:
-            contents: List of contents to retrieve data from
+            content: content identitier
+
+        Returns:
+            associated content's data if any, None otherwise.
 
         """
-        for obj_id in contents:
-            try:
-                data = self.objstorage.get(obj_id)
-            except ObjNotFoundError:
-                yield None
-                continue
+        try:
+            data = self.objstorage.get(obj_id)
+        except ObjNotFoundError:
+            data = None
 
-            yield {'sha1': obj_id, 'data': data}
+        return data
 
     def content_add(self, contents: Iterable[Content]) -> Dict:
         """Add contents to the objstorage.
@@ -55,12 +58,9 @@ class ObjStorage:
         try:
             contents = [c.with_data() for c in contents]
         except MissingData:
-            raise StorageArgumentException('Missing data') from None
-        summary = self.objstorage.add_batch({
-            cont.sha1: cont.data
-            for cont in contents
-        })
+            raise StorageArgumentException("Missing data") from None
+        summary = self.objstorage.add_batch({cont.sha1: cont.data for cont in contents})
         return {
-            'content:add': summary['object:add'],
-            'content:add:bytes': summary['object:add:bytes']
+            "content:add": summary["object:add"],
+            "content:add:bytes": summary["object:add:bytes"],
         }
