@@ -3,15 +3,15 @@
 # License: GNU General Public License version 3, or any later version
 # See top-level LICENSE file for more information
 
-from unittest.mock import patch
-
 import pytest
 
 import swh.storage.api.server as server
-import swh.storage.storage
+import swh.storage
 from swh.storage import get_storage
-from swh.storage.tests.test_storage import TestStorageGeneratedData  # noqa
-from swh.storage.tests.test_storage import TestStorage as _TestStorage
+from swh.storage.tests.storage_tests import (
+    TestStorageGeneratedData as _TestStorageGeneratedData,
+)
+from swh.storage.tests.storage_tests import TestStorage as _TestStorage
 
 # tests are executed using imported classes (TestStorage and
 # TestStorageGeneratedData) using overloaded swh_storage fixture
@@ -62,9 +62,36 @@ def swh_storage(swh_rpc_client, app_server):
     storage.journal_writer = journal_writer
 
 
-class TestStorage(_TestStorage):
-    def test_content_update(self, swh_storage, app_server, sample_data):
-        # TODO, journal_writer not supported
-        swh_storage.journal_writer.journal = None
-        with patch.object(server.storage.journal_writer, "journal", None):
-            super().test_content_update(swh_storage, sample_data)
+class TestStorageApi(_TestStorage):
+    @pytest.mark.skip(
+        'The "person" table of the pgsql is a legacy thing, and not '
+        "supported by the cassandra backend."
+    )
+    def test_person_fullname_unicity(self):
+        pass
+
+    @pytest.mark.skip("content_update is not yet implemented for Cassandra")
+    def test_content_update(self):
+        pass
+
+    @pytest.mark.skip("Not supported by Cassandra")
+    def test_origin_count(self):
+        pass
+
+
+class TestStorageApiGeneratedData(_TestStorageGeneratedData):
+    @pytest.mark.skip("Not supported by Cassandra")
+    def test_origin_count(self):
+        pass
+
+    @pytest.mark.skip("Not supported by Cassandra")
+    def test_origin_count_with_visit_no_visits(self):
+        pass
+
+    @pytest.mark.skip("Not supported by Cassandra")
+    def test_origin_count_with_visit_with_visits_and_snapshot(self):
+        pass
+
+    @pytest.mark.skip("Not supported by Cassandra")
+    def test_origin_count_with_visit_with_visits_no_snapshot(self):
+        pass
