@@ -15,12 +15,12 @@ def get_storage_with_buffer_config(**buffer_config):
     return get_storage(**storage_config)
 
 
-def test_buffering_proxy_storage_content_threshold_not_hit(sample_data_model):
-    contents = sample_data_model["content"]
+def test_buffering_proxy_storage_content_threshold_not_hit(sample_data):
+    contents = sample_data.contents[:2]
     contents_dict = [c.to_dict() for c in contents]
 
     storage = get_storage_with_buffer_config(min_batch_size={"content": 10,})
-    s = storage.content_add([contents[0], contents[1]])
+    s = storage.content_add(contents)
     assert s == {}
 
     # contents have not been written to storage
@@ -37,8 +37,8 @@ def test_buffering_proxy_storage_content_threshold_not_hit(sample_data_model):
     assert list(missing_contents) == []
 
 
-def test_buffering_proxy_storage_content_threshold_nb_hit(sample_data_model):
-    content = sample_data_model["content"][0]
+def test_buffering_proxy_storage_content_threshold_nb_hit(sample_data):
+    content = sample_data.content
     content_dict = content.to_dict()
 
     storage = get_storage_with_buffer_config(min_batch_size={"content": 1,})
@@ -56,8 +56,8 @@ def test_buffering_proxy_storage_content_threshold_nb_hit(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_content_deduplicate(sample_data_model):
-    contents = sample_data_model["content"]
+def test_buffering_proxy_storage_content_deduplicate(sample_data):
+    contents = sample_data.contents[:2]
     storage = get_storage_with_buffer_config(min_batch_size={"content": 2,})
 
     s = storage.content_add([contents[0], contents[0]])
@@ -79,8 +79,8 @@ def test_buffering_proxy_storage_content_deduplicate(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_content_threshold_bytes_hit(sample_data_model):
-    contents = sample_data_model["content"]
+def test_buffering_proxy_storage_content_threshold_bytes_hit(sample_data):
+    contents = sample_data.contents[:2]
     content_bytes_min_batch_size = 2
     storage = get_storage_with_buffer_config(
         min_batch_size={"content": 10, "content_bytes": content_bytes_min_batch_size,}
@@ -101,8 +101,8 @@ def test_buffering_proxy_storage_content_threshold_bytes_hit(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_skipped_content_threshold_not_hit(sample_data_model):
-    contents = sample_data_model["skipped_content"]
+def test_buffering_proxy_storage_skipped_content_threshold_not_hit(sample_data):
+    contents = sample_data.skipped_contents
     contents_dict = [c.to_dict() for c in contents]
     storage = get_storage_with_buffer_config(min_batch_size={"skipped_content": 10,})
     s = storage.skipped_content_add([contents[0], contents[1]])
@@ -119,8 +119,8 @@ def test_buffering_proxy_storage_skipped_content_threshold_not_hit(sample_data_m
     assert list(missing_contents) == []
 
 
-def test_buffering_proxy_storage_skipped_content_threshold_nb_hit(sample_data_model):
-    contents = sample_data_model["skipped_content"]
+def test_buffering_proxy_storage_skipped_content_threshold_nb_hit(sample_data):
+    contents = sample_data.skipped_contents
     storage = get_storage_with_buffer_config(min_batch_size={"skipped_content": 1,})
 
     s = storage.skipped_content_add([contents[0]])
@@ -133,8 +133,8 @@ def test_buffering_proxy_storage_skipped_content_threshold_nb_hit(sample_data_mo
     assert s == {}
 
 
-def test_buffering_proxy_storage_skipped_content_deduplicate(sample_data_model):
-    contents = sample_data_model["skipped_content"][:2]
+def test_buffering_proxy_storage_skipped_content_deduplicate(sample_data):
+    contents = sample_data.skipped_contents[:2]
     storage = get_storage_with_buffer_config(min_batch_size={"skipped_content": 2,})
 
     s = storage.skipped_content_add([contents[0], contents[0]])
@@ -155,42 +155,41 @@ def test_buffering_proxy_storage_skipped_content_deduplicate(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_directory_threshold_not_hit(sample_data_model):
-    directories = sample_data_model["directory"]
+def test_buffering_proxy_storage_directory_threshold_not_hit(sample_data):
+    directory = sample_data.directory
     storage = get_storage_with_buffer_config(min_batch_size={"directory": 10,})
-    s = storage.directory_add([directories[0]])
+    s = storage.directory_add([directory])
     assert s == {}
 
-    directory_id = directories[0].id
-    missing_directories = storage.directory_missing([directory_id])
-    assert list(missing_directories) == [directory_id]
+    missing_directories = storage.directory_missing([directory.id])
+    assert list(missing_directories) == [directory.id]
 
     s = storage.flush()
     assert s == {
         "directory:add": 1,
     }
 
-    missing_directories = storage.directory_missing([directory_id])
+    missing_directories = storage.directory_missing([directory.id])
     assert list(missing_directories) == []
 
 
-def test_buffering_proxy_storage_directory_threshold_hit(sample_data_model):
-    directories = sample_data_model["directory"]
+def test_buffering_proxy_storage_directory_threshold_hit(sample_data):
+    directory = sample_data.directory
     storage = get_storage_with_buffer_config(min_batch_size={"directory": 1,})
-    s = storage.directory_add([directories[0]])
+    s = storage.directory_add([directory])
     assert s == {
         "directory:add": 1,
     }
 
-    missing_directories = storage.directory_missing([directories[0].id])
+    missing_directories = storage.directory_missing([directory.id])
     assert list(missing_directories) == []
 
     s = storage.flush()
     assert s == {}
 
 
-def test_buffering_proxy_storage_directory_deduplicate(sample_data_model):
-    directories = sample_data_model["directory"][:2]
+def test_buffering_proxy_storage_directory_deduplicate(sample_data):
+    directories = sample_data.directories[:2]
     storage = get_storage_with_buffer_config(min_batch_size={"directory": 2,})
 
     s = storage.directory_add([directories[0], directories[0]])
@@ -211,42 +210,41 @@ def test_buffering_proxy_storage_directory_deduplicate(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_revision_threshold_not_hit(sample_data_model):
-    revisions = sample_data_model["revision"]
+def test_buffering_proxy_storage_revision_threshold_not_hit(sample_data):
+    revision = sample_data.revision
     storage = get_storage_with_buffer_config(min_batch_size={"revision": 10,})
-    s = storage.revision_add([revisions[0]])
+    s = storage.revision_add([revision])
     assert s == {}
 
-    revision_id = revisions[0].id
-    missing_revisions = storage.revision_missing([revision_id])
-    assert list(missing_revisions) == [revision_id]
+    missing_revisions = storage.revision_missing([revision.id])
+    assert list(missing_revisions) == [revision.id]
 
     s = storage.flush()
     assert s == {
         "revision:add": 1,
     }
 
-    missing_revisions = storage.revision_missing([revision_id])
+    missing_revisions = storage.revision_missing([revision.id])
     assert list(missing_revisions) == []
 
 
-def test_buffering_proxy_storage_revision_threshold_hit(sample_data_model):
-    revisions = sample_data_model["revision"]
+def test_buffering_proxy_storage_revision_threshold_hit(sample_data):
+    revision = sample_data.revision
     storage = get_storage_with_buffer_config(min_batch_size={"revision": 1,})
-    s = storage.revision_add([revisions[0]])
+    s = storage.revision_add([revision])
     assert s == {
         "revision:add": 1,
     }
 
-    missing_revisions = storage.revision_missing([revisions[0].id])
+    missing_revisions = storage.revision_missing([revision.id])
     assert list(missing_revisions) == []
 
     s = storage.flush()
     assert s == {}
 
 
-def test_buffering_proxy_storage_revision_deduplicate(sample_data_model):
-    revisions = sample_data_model["revision"][:2]
+def test_buffering_proxy_storage_revision_deduplicate(sample_data):
+    revisions = sample_data.revisions[:2]
     storage = get_storage_with_buffer_config(min_batch_size={"revision": 2,})
 
     s = storage.revision_add([revisions[0], revisions[0]])
@@ -267,8 +265,8 @@ def test_buffering_proxy_storage_revision_deduplicate(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_release_threshold_not_hit(sample_data_model):
-    releases = sample_data_model["release"]
+def test_buffering_proxy_storage_release_threshold_not_hit(sample_data):
+    releases = sample_data.releases
     threshold = 10
 
     assert len(releases) < threshold
@@ -291,8 +289,8 @@ def test_buffering_proxy_storage_release_threshold_not_hit(sample_data_model):
     assert list(missing_releases) == []
 
 
-def test_buffering_proxy_storage_release_threshold_hit(sample_data_model):
-    releases = sample_data_model["release"]
+def test_buffering_proxy_storage_release_threshold_hit(sample_data):
+    releases = sample_data.releases
     threshold = 2
     assert len(releases) > threshold
 
@@ -313,8 +311,8 @@ def test_buffering_proxy_storage_release_threshold_hit(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_release_deduplicate(sample_data_model):
-    releases = sample_data_model["release"][:2]
+def test_buffering_proxy_storage_release_deduplicate(sample_data):
+    releases = sample_data.releases[:2]
     storage = get_storage_with_buffer_config(min_batch_size={"release": 2,})
 
     s = storage.release_add([releases[0], releases[0]])
@@ -335,20 +333,20 @@ def test_buffering_proxy_storage_release_deduplicate(sample_data_model):
     assert s == {}
 
 
-def test_buffering_proxy_storage_clear(sample_data_model):
+def test_buffering_proxy_storage_clear(sample_data):
     """Clear operation on buffer
 
     """
     threshold = 10
-    contents = sample_data_model["content"]
+    contents = sample_data.contents
     assert 0 < len(contents) < threshold
-    skipped_contents = sample_data_model["skipped_content"]
+    skipped_contents = sample_data.skipped_contents
     assert 0 < len(skipped_contents) < threshold
-    directories = sample_data_model["directory"]
+    directories = sample_data.directories
     assert 0 < len(directories) < threshold
-    revisions = sample_data_model["revision"]
+    revisions = sample_data.revisions
     assert 0 < len(revisions) < threshold
-    releases = sample_data_model["release"]
+    releases = sample_data.releases
     assert 0 < len(releases) < threshold
 
     storage = get_storage_with_buffer_config(
