@@ -94,15 +94,15 @@ def test_filtering_proxy_storage_skipped_content_missing_sha1_git(
 def test_filtering_proxy_storage_revision(swh_storage, sample_data):
     sample_revision = sample_data.revision
 
-    revision = next(swh_storage.revision_get([sample_revision.id]))
-    assert not revision
+    revision = swh_storage.revision_get([sample_revision.id])[0]
+    assert revision is None
 
     s = swh_storage.revision_add([sample_revision])
     assert s == {
         "revision:add": 1,
     }
 
-    revision = next(swh_storage.revision_get([sample_revision.id]))
+    revision = swh_storage.revision_get([sample_revision.id])[0]
     assert revision is not None
 
     s = swh_storage.revision_add([sample_revision])
@@ -114,7 +114,7 @@ def test_filtering_proxy_storage_revision(swh_storage, sample_data):
 def test_filtering_proxy_storage_directory(swh_storage, sample_data):
     sample_directory = sample_data.directory
 
-    directory = next(swh_storage.directory_missing([sample_directory.id]))
+    directory = list(swh_storage.directory_missing([sample_directory.id]))[0]
     assert directory
 
     s = swh_storage.directory_add([sample_directory])
