@@ -13,6 +13,7 @@ import pytest
 from swh.model.model import MetadataTargetType
 
 from swh.storage.exc import HashCollision, StorageArgumentException
+from swh.storage.utils import now
 
 
 @pytest.fixture
@@ -120,7 +121,7 @@ def test_retrying_proxy_storage_content_add_metadata(swh_storage, sample_data):
     content_metadata = swh_storage.content_get([pk])
     assert content_metadata == [None]
 
-    s = swh_storage.content_add_metadata([content])
+    s = swh_storage.content_add_metadata([attr.evolve(content, ctime=now())])
     assert s == {
         "content:add": 1,
     }
@@ -619,16 +620,16 @@ def test_retrying_proxy_storage_revision_add(swh_storage, sample_data):
     """
     sample_rev = sample_data.revision
 
-    revision = next(swh_storage.revision_get([sample_rev.id]))
-    assert not revision
+    revision = swh_storage.revision_get([sample_rev.id])[0]
+    assert revision is None
 
     s = swh_storage.revision_add([sample_rev])
     assert s == {
         "revision:add": 1,
     }
 
-    revision = next(swh_storage.revision_get([sample_rev.id]))
-    assert revision["id"] == sample_rev.id
+    revision = swh_storage.revision_get([sample_rev.id])[0]
+    assert revision == sample_rev
 
 
 def test_retrying_proxy_storage_revision_add_with_retry(
@@ -649,8 +650,8 @@ def test_retrying_proxy_storage_revision_add_with_retry(
 
     sample_rev = sample_data.revision
 
-    revision = next(swh_storage.revision_get([sample_rev.id]))
-    assert not revision
+    revision = swh_storage.revision_get([sample_rev.id])[0]
+    assert revision is None
 
     s = swh_storage.revision_add([sample_rev])
     assert s == {
@@ -673,8 +674,8 @@ def test_retrying_proxy_swh_storage_revision_add_failure(
 
     sample_rev = sample_data.revision
 
-    revision = next(swh_storage.revision_get([sample_rev.id]))
-    assert not revision
+    revision = swh_storage.revision_get([sample_rev.id])[0]
+    assert revision is None
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
         swh_storage.revision_add([sample_rev])
@@ -688,16 +689,16 @@ def test_retrying_proxy_storage_release_add(swh_storage, sample_data):
     """
     sample_rel = sample_data.release
 
-    release = next(swh_storage.release_get([sample_rel.id]))
-    assert not release
+    release = swh_storage.release_get([sample_rel.id])[0]
+    assert release is None
 
     s = swh_storage.release_add([sample_rel])
     assert s == {
         "release:add": 1,
     }
 
-    release = next(swh_storage.release_get([sample_rel.id]))
-    assert release["id"] == sample_rel.id
+    release = swh_storage.release_get([sample_rel.id])[0]
+    assert release == sample_rel
 
 
 def test_retrying_proxy_storage_release_add_with_retry(
@@ -718,8 +719,8 @@ def test_retrying_proxy_storage_release_add_with_retry(
 
     sample_rel = sample_data.release
 
-    release = next(swh_storage.release_get([sample_rel.id]))
-    assert not release
+    release = swh_storage.release_get([sample_rel.id])[0]
+    assert release is None
 
     s = swh_storage.release_add([sample_rel])
     assert s == {
@@ -742,8 +743,8 @@ def test_retrying_proxy_swh_storage_release_add_failure(
 
     sample_rel = sample_data.release
 
-    release = next(swh_storage.release_get([sample_rel.id]))
-    assert not release
+    release = swh_storage.release_get([sample_rel.id])[0]
+    assert release is None
 
     with pytest.raises(StorageArgumentException, match="Refuse to add"):
         swh_storage.release_add([sample_rel])
